@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011-2012 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 
 """Handle fields on screen."""
 
@@ -62,6 +62,7 @@ MPD_TO_LCDD_MAP = {
 
 class Field(object):
     base_name = None
+    target_hooks = []
 
     def __init__(self, ref, width=-1, **kwargs):
         assert self.base_name
@@ -76,8 +77,31 @@ class Field(object):
         return self.width < 0
 
     def add_to_screen(self, screen, left, top):
+        """Add the field to the screen.
+
+        The 'screen' object is a lcdproc Screen object, and left/top indicate
+        the topleft corner attributed to the field by the global pattern.
+
+        This method must return the generated widget.
+        """
         raise NotImplementedError()
 
+    def register_hooks(self):
+        """Register hooks to be notified of field changes.
+
+        Should return a list of fields whose information is watched.
+        """
+        return self.target_hooks
+
+    def hook_changed(self, hook_name, widget, new_data):
+        if hook_name == 'song':
+            self.song_changed(widget, new_data)
+        elif hook_name == 'state':
+            self.state_changed(widget, new_data)
+        elif hook_name == 'elapsed_and_total':
+            elapsed, total = new_data
+            self.time_changed(widget, elapsed, total)
+
     def song_changed(self, widget, new_song):
         pass
 
@@ -116,6 +140,7 @@ class FixedText(Field):
 @register_field
 class StateField(Field):
     base_name = 'state'
+    target_hooks = ['state']
 
     def __init__(self, **kwargs):
         super(StateField, self).__init__(width=1, **kwargs)
@@ -130,6 +155,8 @@ class StateField(Field):
 
 
 class BaseTimeField(Field):
+    target_hooks = ['state', 'elapsed_and_total']
+
     def __init__(self, **kwargs):
         super(BaseTimeField, self).__init__(width=5, **kwargs)
 
@@ -178,14 +205,47 @@ class RemainingTimeField(BaseTimeField):
     base_name = 'remaining'
 
     def time_changed(self, widget, elapsed, total):
-        txt = self._format_time(total - elapsed)
+        if total is not None and elapsed is not None:
+            remaining = total - elapsed
+        else:
+            remaining = None
+        txt = self._format_time(remaining)
         logger.debug(u'Setting widget %s to %r', widget.ref, txt)
         widget.set_text(txt)
 
 
+@register_field
+class BitRateField(Field):
+    base_name = 'bitrate'
+    target_hooks = ['status']
+
+    def _format_bitrate(self, bitrate=0):
+        bitrate = int(bitrate)
+        return '%3d' % bitrate
+
+    def __init__(self, **kwargs):
+        width = len(self._format_bitrate())
+        super(BitRateField, self).__init__(width=width, **kwargs)
+
+    def add_to_screen(self, screen, left, top):
+        return screen.add_string_widget(self.name,
+            self._format_bitrate(), x=left, y=top)
+
+    def hook_changed(self, hook_name, widget, new_data):
+        if hook_name == 'status':
+            self.status_changed(widget, new_data)
+        super(BitRateField, self).hook_changed(hook_name, widget, new_data)
+
+    def status_changed(self, widget, new_status):
+        txt = self._format_bitrate(new_status.get('bitrate') or 0)
+        logger.debug(u"Setting widget %r to %r", widget.ref, txt)
+        widget.set_text(txt)
+
+
 @register_field
 class SongField(Field):
     base_name = 'song'
+    target_hooks = ['song']
 
     def __init__(self, format=u'', width=-1, speed=2, **kwargs):
         self.format = format
@@ -200,12 +260,7 @@ class SongField(Field):
     @classmethod
     def _song_dict(cls, song):
         fields = collections.defaultdict(lambda: u'')
-        fields.update({
-            'artist': song.artist,
-            'title': song.title,
-            'album': song.album,
-            'duration': song.time,
-        })
+        fields.update(song.tags)
         return fields
 
     def song_changed(self, widget, new_song):
